@@ -1,11 +1,12 @@
+import contextlib
 import contextvars
 import datetime
 import json
-from time import time
+from time import perf_counter
 
+import django.test.testcases
 from django.utils.encoding import force_str
 
-from debug_toolbar import settings as dt_settings
 from debug_toolbar.utils import get_stack_trace, get_template_info
 
 try:
@@ -31,105 +32,83 @@ class SQLQueryTriggered(Exception):
     """Thrown when template panel triggers a query"""
 
 
-def wrap_cursor(connection, panel):
+def wrap_cursor(connection):
+    # When running a SimpleTestCase, Django monkey patches some DatabaseWrapper
+    # methods, including .cursor() and .chunked_cursor(), to raise an exception
+    # if the test code tries to access the database, and then undoes the monkey
+    # patching when the test case is finished.  If we monkey patch those methods
+    # also, Django's process of undoing those monkey patches will fail.  To
+    # avoid this failure, and because database access is not allowed during a
+    # SimpleTestCase anyway, skip applying our instrumentation monkey patches if
+    # we detect that Django has already monkey patched DatabaseWrapper.cursor().
+    if isinstance(connection.cursor, django.test.testcases._DatabaseFailure):
+        return
     if not hasattr(connection, "_djdt_cursor"):
         connection._djdt_cursor = connection.cursor
         connection._djdt_chunked_cursor = connection.chunked_cursor
+        connection._djdt_logger = None
 
         def cursor(*args, **kwargs):
             # Per the DB API cursor() does not accept any arguments. There's
             # some code in the wild which does not follow that convention,
             # so we pass on the arguments even though it's not clean.
             # See:
-            # https://github.com/jazzband/django-debug-toolbar/pull/615
-            # https://github.com/jazzband/django-debug-toolbar/pull/896
-            if allow_sql.get():
-                wrapper = NormalCursorWrapper
-            else:
-                wrapper = ExceptionCursorWrapper
-            return wrapper(connection._djdt_cursor(*args, **kwargs), connection, panel)
+            # https://github.com/django-commons/django-debug-toolbar/pull/615
+            # https://github.com/django-commons/django-debug-toolbar/pull/896
+            logger = connection._djdt_logger
+            cursor = connection._djdt_cursor(*args, **kwargs)
+            if logger is None:
+                return cursor
+            mixin = NormalCursorMixin if allow_sql.get() else ExceptionCursorMixin
+            return patch_cursor_wrapper_with_mixin(cursor.__class__, mixin)(
+                cursor.cursor, connection, logger
+            )
 
         def chunked_cursor(*args, **kwargs):
             # prevent double wrapping
-            # solves https://github.com/jazzband/django-debug-toolbar/issues/1239
+            # solves https://github.com/django-commons/django-debug-toolbar/issues/1239
+            logger = connection._djdt_logger
             cursor = connection._djdt_chunked_cursor(*args, **kwargs)
-            if not isinstance(cursor, BaseCursorWrapper):
-                if allow_sql.get():
-                    wrapper = NormalCursorWrapper
-                else:
-                    wrapper = ExceptionCursorWrapper
-                return wrapper(cursor, connection, panel)
+            if logger is not None and not isinstance(cursor, DjDTCursorWrapperMixin):
+                mixin = NormalCursorMixin if allow_sql.get() else ExceptionCursorMixin
+                return patch_cursor_wrapper_with_mixin(cursor.__class__, mixin)(
+                    cursor.cursor, connection, logger
+                )
             return cursor
 
         connection.cursor = cursor
         connection.chunked_cursor = chunked_cursor
-        return cursor
 
 
-def unwrap_cursor(connection):
-    if hasattr(connection, "_djdt_cursor"):
-        # Sometimes the cursor()/chunked_cursor() methods of the DatabaseWrapper
-        # instance are already monkey patched before wrap_cursor() is called.  (In
-        # particular, Django's SimpleTestCase monkey patches those methods for any
-        # disallowed databases to raise an exception if they are accessed.)  Thus only
-        # delete our monkey patch if the method we saved is the same as the class
-        # method.  Otherwise, restore the prior monkey patch from our saved method.
-        if connection._djdt_cursor == connection.__class__.cursor:
-            del connection.cursor
-        else:
-            connection.cursor = connection._djdt_cursor
-        del connection._djdt_cursor
-        if connection._djdt_chunked_cursor == connection.__class__.chunked_cursor:
-            del connection.chunked_cursor
-        else:
-            connection.chunked_cursor = connection._djdt_chunked_cursor
-        del connection._djdt_chunked_cursor
+def patch_cursor_wrapper_with_mixin(base_wrapper, mixin):
+    class DjDTCursorWrapper(mixin, base_wrapper):
+        pass
+
+    return DjDTCursorWrapper
 
 
-class BaseCursorWrapper:
-    pass
+class DjDTCursorWrapperMixin:
+    def __init__(self, cursor, db, logger):
+        super().__init__(cursor, db)
+        # logger must implement a ``record`` method
+        self.logger = logger
 
 
-class ExceptionCursorWrapper(BaseCursorWrapper):
+class ExceptionCursorMixin(DjDTCursorWrapperMixin):
     """
     Wraps a cursor and raises an exception on any operation.
     Used in Templates panel.
     """
 
-    def __init__(self, cursor, db, logger):
-        pass
-
     def __getattr__(self, attr):
         raise SQLQueryTriggered()
 
 
-class NormalCursorWrapper(BaseCursorWrapper):
+class NormalCursorMixin(DjDTCursorWrapperMixin):
     """
     Wraps a cursor and logs queries.
     """
 
-    def __init__(self, cursor, db, logger):
-        self.cursor = cursor
-        # Instance of a BaseDatabaseWrapper subclass
-        self.db = db
-        # logger must implement a ``record`` method
-        self.logger = logger
-
-    def _quote_expr(self, element):
-        if isinstance(element, str):
-            return "'%s'" % element.replace("'", "''")
-        else:
-            return repr(element)
-
-    def _quote_params(self, params):
-        if not params:
-            return params
-        if isinstance(params, dict):
-            return {key: self._quote_expr(value) for key, value in params.items()}
-        if isinstance(params, tuple):
-            return tuple(self._quote_expr(p) for p in params)
-        return [self._quote_expr(p) for p in params]
-
     def _decode(self, param):
         if PostgresJson and isinstance(param, PostgresJson):
             # psycopg3
@@ -154,6 +133,19 @@ class NormalCursorWrapper(BaseCursorWrapper):
         except UnicodeDecodeError:
             return "(encoded string)"
 
+    def _last_executed_query(self, sql, params):
+        """Get the last executed query from the connection."""
+        # Django's psycopg3 backend creates a new cursor in its implementation of the
+        # .last_executed_query() method.  To avoid wrapping that cursor, temporarily set
+        # the DatabaseWrapper's ._djdt_logger attribute to None.  This will cause the
+        # monkey-patched .cursor() and .chunked_cursor() methods to skip the wrapping
+        # process during the .last_executed_query() call.
+        self.db._djdt_logger = None
+        try:
+            return self.db.ops.last_executed_query(self.cursor, sql, params)
+        finally:
+            self.db._djdt_logger = self.logger
+
     def _record(self, method, sql, params):
         alias = self.db.alias
         vendor = self.db.vendor
@@ -163,41 +155,36 @@ class NormalCursorWrapper(BaseCursorWrapper):
             conn = self.db.connection
             initial_conn_status = conn.info.transaction_status
 
-        start_time = time()
+        start_time = perf_counter()
         try:
             return method(sql, params)
         finally:
-            stop_time = time()
+            stop_time = perf_counter()
             duration = (stop_time - start_time) * 1000
             _params = ""
-            try:
+            with contextlib.suppress(TypeError):
+                # object JSON serializable?
                 _params = json.dumps(self._decode(params))
-            except TypeError:
-                pass  # object not JSON serializable
             template_info = get_template_info()
 
             # Sql might be an object (such as psycopg Composed).
             # For logging purposes, make sure it's str.
             if vendor == "postgresql" and not isinstance(sql, str):
-                sql = sql.as_string(conn)
+                if isinstance(sql, bytes):
+                    sql = sql.decode("utf-8")
+                else:
+                    sql = sql.as_string(conn)
             else:
                 sql = str(sql)
 
-            params = {
+            kwargs = {
                 "vendor": vendor,
                 "alias": alias,
-                "sql": self.db.ops.last_executed_query(
-                    self.cursor, sql, self._quote_params(params)
-                ),
+                "sql": self._last_executed_query(sql, params),
                 "duration": duration,
                 "raw_sql": sql,
                 "params": _params,
-                "raw_params": params,
                 "stacktrace": get_stack_trace(skip=2),
-                "start_time": start_time,
-                "stop_time": stop_time,
-                "is_slow": duration > dt_settings.get_config()["SQL_WARNING_THRESHOLD"],
-                "is_select": sql.lower().strip().startswith("select"),
                 "template_info": template_info,
             }
 
@@ -229,7 +216,7 @@ class NormalCursorWrapper(BaseCursorWrapper):
                 else:
                     trans_id = None
 
-                params.update(
+                kwargs.update(
                     {
                         "trans_id": trans_id,
                         "trans_status": conn.info.transaction_status,
@@ -238,25 +225,13 @@ class NormalCursorWrapper(BaseCursorWrapper):
                 )
 
             # We keep `sql` to maintain backwards compatibility
-            self.logger.record(**params)
+            self.logger.record(**kwargs)
 
     def callproc(self, procname, params=None):
-        return self._record(self.cursor.callproc, procname, params)
+        return self._record(super().callproc, procname, params)
 
     def execute(self, sql, params=None):
-        return self._record(self.cursor.execute, sql, params)
+        return self._record(super().execute, sql, params)
 
     def executemany(self, sql, param_list):
-        return self._record(self.cursor.executemany, sql, param_list)
-
-    def __getattr__(self, attr):
-        return getattr(self.cursor, attr)
-
-    def __iter__(self):
-        return iter(self.cursor)
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, type, value, traceback):
-        self.close()
+        return self._record(super().executemany, sql, param_list)
