@@ -1,3 +1,4 @@
+import os
 import sys
 import warnings
 from functools import cache
@@ -6,6 +7,15 @@ from django.conf import settings
 from django.dispatch import receiver
 from django.test.signals import setting_changed
 
+
+def _is_running_tests():
+    """
+    Helper function to support testing default value for
+    IS_RUNNING_TESTS
+    """
+    return "test" in sys.argv or "PYTEST_VERSION" in os.environ
+
+
 CONFIG_DEFAULTS = {
     # Toolbar options
     "DISABLE_PANELS": {
@@ -43,9 +53,9 @@ CONFIG_DEFAULTS = {
     "SQL_WARNING_THRESHOLD": 500,  # milliseconds
     "OBSERVE_REQUEST_CALLBACK": "debug_toolbar.toolbar.observe_request",
     "TOOLBAR_LANGUAGE": None,
-    "IS_RUNNING_TESTS": "test" in sys.argv,
+    "TOOLBAR_STORE_CLASS": "debug_toolbar.store.MemoryStore",
+    "IS_RUNNING_TESTS": _is_running_tests(),
     "UPDATE_ON_FETCH": False,
-    "DEFAULT_THEME": "auto",
 }
 
 
